-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : ser_in_tb.vhd
-- Revision : V0.1
-- Author :   Zdenk Sojka
-- Created :  09. 05. 2007
--
-- Modifications
-- 0.1	09. 05. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- zakladni testbench na vstup seriove linky
--
-- 0.1	prvn verze
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY ser_in_tb_vhd IS
END ser_in_tb_vhd;

ARCHITECTURE behavior OF ser_in_tb_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT ser_in
	PORT(
		rx : IN std_logic;
		clk : IN std_logic;
		ack : IN std_logic;          
		rst : IN std_logic;          
		data : OUT std_logic_vector(7 downto 0);
		rdy : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL rx :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '1';
	SIGNAL ack :  std_logic := '0';

	--Outputs
	SIGNAL data :  std_logic_vector(7 downto 0);
	SIGNAL rdy :  std_logic;

	SIGNAL iclk :  std_logic := '0';

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: ser_in PORT MAP(
		rx => rx,
		data => data,
		rdy => rdy,
		ack => ack,
		clk => clk,
		rst => rst
	);
	
	inhod : process	-- casovani odesilani
	begin
		iclk <= '1';
		wait for 26 us;
		iclk <= '0';
		wait for 26 us;
	end process;
	
	hodiny : process	-- hodiny
	begin
		clk <= '1';
		wait for 10ns;	-- pro 50 MHz hodiny
		clk <= '0';
		wait for 10ns;
	end process;

	tb : PROCESS
	BEGIN
	
		rst<='1';
		rx<='1';
		-- Wait 100 ns for global reset to finish
		wait for 60 ns;
		rst<='0';
		wait for 40 ns;
		wait until iclk'event and (iclk = '0');
		rx <= '0';
		
		wait until iclk'event and (iclk = '0');	-- jednotlive bity...
		rx <= '0';
		wait until iclk'event and (iclk = '0');
		wait until iclk'event and (iclk = '0');

		rx <= '1';
		wait until iclk'event and (iclk = '0');
		wait until iclk'event and (iclk = '0');
		wait until iclk'event and (iclk = '0');
		rx <= '0';
		wait until iclk'event and (iclk = '0');
		wait until iclk'event and (iclk = '0');
		rx <= '1';
		wait until iclk'event and (iclk = '0');
		rx <= '1';
		wait for 40 ns;
		assert ( data = "10011100" );
		ack <= '1';
		wait for 1 us;
		ack <= '0';

		-- Place stimulus here
		wait; -- will wait forever
	END PROCESS;

END;
