-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : top_tb_f.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  16. 05. 2007
--
-- Modifications
-- 1.0	20. 05. 2007 by Zdenk Sojka
-- 0.1	16. 05. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- testbench celho ppravku
-- 
-- soubor "tb_f_verode.txt" - odesilane znaky
-- soubor "tb_f_verpri.txt" - predpokladane prijimane znaky
--
-- 0.1	prvn verze
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
USE std.textio.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY top_tb_f_vhd IS
END top_tb_f_vhd;

ARCHITECTURE behavior OF top_tb_f_vhd IS 

--	constant iot : time := 0.5 us;	-- 1 mbit
--	constant iot : time := 4.34 us;	-- 115200 bps
	constant iot : time := 26.04 us;	-- 19200 bps
--	constant iot : time := 1.67 ms;	-- 300 bps

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT top
	PORT(
		rx : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		tx : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL rx :  std_logic := '1';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';

	--Outputs
	SIGNAL tx :  std_logic;

	signal iclk : std_logic;
	signal posilat : std_logic := '0';
	
	file verode : text is "tb_f_verode.txt";
	file verpri : text is "tb_f_verpri.txt";

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: top PORT MAP(
		rx => rx,
		tx => tx,
		clk => clk,
		rst => rst
	);

	iohod : process	-- casovani odesilani
	begin
		iclk <= '1';
		wait for iot;
		iclk <= '0';
		wait for iot;
	end process;

	hodiny : process	-- hodiny
	begin
		clk <= '1';
		wait for 10ns;	-- pro 50 MHz hodiny
		clk <= '0';
		wait for 10ns;
	end process;

	odesilani : process	-- odesilani souboru do seriove linky
		variable cek : std_logic_vector(7 downto 0);
		variable poslat : std_logic_vector(10 downto 0);
		variable radekTX : line;
		variable znak : character;
		begin
			wait until posilat = '1';
			while not endfile(verode) loop
				readline(verode,radekTX);
				for j in radekTX'range loop
					read(radekTX,znak);
					cek := CONV_STD_LOGIC_VECTOR(character'pos(znak),8);
					poslat := "11" & cek & "0";
					for i in poslat'reverse_range loop
						wait until iclk'event and (iclk = '0');
						rx <= poslat(i);
					end loop;
				end loop;
				wait for 3 ms;
			end loop;
	end process;
	
	prijimani : process	-- nepravdiva informace, ze neni wait - wait je ve volane procedure !!!
								-- prijima znaky z pripravku, porovnava je s daty ctenymi ze souboru
		procedure prijmout(variable cek: in std_logic_vector(7 downto 0)) is		-- precte jeden znak a porovna s predpokladanym
	
		begin
			wait until tx = '0';
			wait for iot;
			assert( tx = '0' );		-- start bit
			for i in 0 to 7 loop		-- od nejnizsiho bitu
				wait for 2 * iot;
				assert( tx = cek(i) );
			end loop;
			wait for 2 * iot;
			assert( tx = '1' );	-- jeden stop bit
			wait for 2 * iot;
			assert( tx = '1' );	-- druhy stop bit
		end procedure;

		variable cek : std_logic_vector(7 downto 0);
		variable radekRX : line;
		variable znak : character;
		begin
			while not endfile(verpri) loop
				readline(verpri,radekRX);
				for j in radekRX'range loop
					read(radekRX,znak);
					cek := CONV_STD_LOGIC_VECTOR(character'pos(znak),8);
					prijmout(cek => cek);
				end loop;

				cek := "00001101";
				prijmout( cek => cek );
				cek := "00001010";
				prijmout( cek => cek );
			end loop;
	end process;

	tb : PROCESS
	BEGIN
		rst <= '1';
		-- Wait 100 ns for global reset to finish
		wait for 40 ns;
		rst <= '0';
		wait for 40 ns;
		posilat <= '1';
		wait; -- will wait forever
	END PROCESS;

END;
