-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : top_tb_j.vhd
-- Revision : V1.0
-- Author :   Josef Nouzak
-- Created :  20. 05. 2007
--
-- Modifications
-- 1.0	20. 05. 2007 by Josef Nouzak
-----------------------------------------------------------------
-- testbench celeho projektu (entity top)
--
-- soubor "tb_j_data_in.txt" obsahuje leve strany rovnice
-- soubor "tb_j_vysledek.txt" obsahuje prave strany
--
-- 1.0	initial release
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;
USE ieee.std_logic_arith.all;

ENTITY top_tb_j_vhd IS
END top_tb_j_vhd;

ARCHITECTURE behavior OF top_tb_j_vhd IS 

--	constant iot : time := 0.5 us;	-- 1 mbit
--	constant iot : time := 4.34 us;	-- 115200 bps
	constant iot : time := 26.04 us;	-- 19200 bps
--	constant iot : time := 1.67 ms;	-- 300 bps



	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT top
	PORT(
		rx : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		tx : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL rx :  std_logic := '1';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';

	--Outputs
	SIGNAL tx :  std_logic;

constant stopbits : natural := 2;
signal rxdata    : std_logic_vector(7 downto 0);   -- prijata data (z uut)
signal receiveReq : std_logic := '0';   -- pozadavek na prijeti znaku
signal receiveAck : std_logic := '0';   -- potvrzeni prijeti znaku
signal txdata : std_logic_vector(7 downto 0);   -- data k odeslani (do uut)
signal transmitReq : std_logic := '0';   -- pozadavek na odeslani znaku
signal transmitAck : std_logic := '0';   -- potvrzeni odeslani znaku

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: top PORT MAP(
		rx => rx,
		tx => tx,
		clk => clk,
		rst => rst
	);
 
receive : process   -- prijeti znaku pres seriovou linku (vysledek v promenne rxdata)
 begin
	wait until receiveReq'event and receiveReq = '1';   -- cekani na pozadavek
	wait until tx'event and tx = '0';   -- cekani na startbit
	wait for iot;   -- mereni kazdeho bitu bude v polovine jeho sirky (posun o pulperiodu)s
	for i in rxdata'reverse_range loop
		wait for iot; wait for iot;
		rxdata(i) <= tx;   -- uzitecna data
	end loop;
	for i in 1 to stopbits loop   -- kontrola stopbitu
		wait for 2 * iot;   -- perioda bitu;
		assert( tx = '1' );
	end loop;
	receiveAck <= '1';
	receiveAck <= '0' after iot;
 end process;

transmit : process   -- odeslani znaku pres seriovou linku (data v promenne txdata)
 begin
	wait until transmitReq'event and transmitReq = '1';   -- cekani na pozadavek
	rx <= '0';   -- startbit;
	wait for iot; wait for iot;
	for i in txdata'reverse_range loop
		rx <= txdata(i);   -- odeslani bitu
		wait for iot; wait for iot;
	end loop;
	for i in 1 to stopbits loop   -- odeslani stopbitu
		rx <= '1';
		wait for 2 * iot;   -- perioda bitu
	end loop;
	rx <= '1';   -- klidova uroven
	transmitAck <= '1';
	transmitAck <= '0' after iot;
 end process;

	tb : process
	 	
		FILE data : TEXT is IN "tb_j_data_in.txt"; --data pro vstup
	
		VARIABLE znak : character;
		VARIABLE radek : line;
		
		
		FILE data2 : TEXT is IN "tb_j_vysledek.txt"; --data pro vystup
		VARIABLE znak2 : character;
		VARIABLE radek2 : line;
	 
	 
	 begin

		rst <= '1';
		wait for 1ns;
		rst <= '0';
		
		
		
		WHILE not endfile(data) LOOP
			
			readline(data, radek);
			readline(data2, radek2);
			
			for j in 0 to 5 loop
			read(radek,znak);		
			txdata <= CONV_STD_LOGIC_VECTOR(character'pos(znak),8);
			transmitReq <= '1', '0' after iot;
			receiveReq <= '1', '0' after iot;
			wait until rising_edge(transmitAck);
			wait until rising_edge(receiveAck);
			assert( rxdata = CONV_STD_LOGIC_VECTOR(character'pos(znak),8));
			end loop;
			receiveReq <= '1', '0' after iot;
			wait until rising_edge(receiveAck);
			read(radek2,znak2);
			assert( rxdata = CONV_STD_LOGIC_VECTOR(character'pos(znak2),8));
			receiveReq <= '1', '0' after iot;
			wait until rising_edge(receiveAck);
			read(radek2,znak2);
			assert( rxdata = CONV_STD_LOGIC_VECTOR(character'pos(znak2),8));
			receiveReq <= '1', '0' after iot;
			wait until rising_edge(receiveAck);
			read(radek2,znak2);
			assert( rxdata = CONV_STD_LOGIC_VECTOR(character'pos(znak2),8));
			receiveReq <= '1', '0' after iot; --odradkovani 
			wait until rising_edge(receiveAck);
			assert( rxdata = std_logic_vector(to_unsigned(13, 8)) );
			wait for 3ms;
		END LOOP;
		
	
		report "Testbench finished, check for failed assertions" severity note;
		wait;
	 end process;
	
	clock : process
	 begin
		clk <= '0';
		wait for 10ns;	-- pro 50 MHz hodiny
		clk <= '1';
		wait for 10ns;
	 end process;

END;