-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : bridge.vhd
-- Revision : V1.0
-- Author :   Josef Nouzk
-- Created :  13. 05. 2007
--
-- Modifications
-- 1.0	13. 05. 2007 by Josef Nouzk
-----------------------------------------------------------------
-- picoBlaze softcore s kdem hlavn obsluhy
--	komunikace s PC a pedvn dat nsobice
--
-- vstupy	: data_rx, act_uart, vys, valid, clk, rst, rdy_uart
-- vstupy	: data_tx, data_mul, ack_uart, send_uart, btns
--
-- 1.0	initial release
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity bridge is
	port (
		data_rx : in std_logic_vector(7 downto 0);   -- vstup z UART
		data_tx : out std_logic_vector(7 downto 0);   -- vystup do UART
		data_mul : out std_logic_vector(7 downto 0);   -- vystup do nasobicky
		act_uart : in std_logic;   -- aktivni odesilani UART
		ack_uart : out std_logic;   -- potvrzeni prijmu dat UART
		send_uart : out std_logic;   -- strobovani pro odeslani  UART
		rdy_uart : in std_logic;   -- prijmuta data UART
		btns : out std_logic_vector(1 downto 0);	-- tlactka k potvrzovani operandu nasobicky
		valid : in std_logic;   -- potvrzeni platnosti dat na vystupu nasobicky
		vys : in std_logic_vector(15 downto 0);	-- souin		
		
		clk : in std_logic;
		rst : in std_logic);
		
end entity;

architecture rtl of bridge is

signal port_id, out_port, in_port : std_logic_vector(7 downto 0);
signal write_strobe, read_strobe : std_logic;

component embedded_kcpsm_brg is
    Port (      port_id : out std_logic_vector(7 downto 0);
           write_strobe : out std_logic;
            read_strobe : out std_logic;
               out_port : out std_logic_vector(7 downto 0);
                in_port : in std_logic_vector(7 downto 0);
              interrupt : in std_logic;
                  reset : in std_logic;
                    clk : in std_logic);
end component;

begin

	most : embedded_kcpsm_brg
	port map (
		port_id => port_id,
		write_strobe => write_strobe,
		read_strobe => read_strobe,
		out_port => out_port,
		in_port => in_port,
		interrupt => '0',
		reset => rst,
		clk => clk
	);
	
-- buffering used output ports
data_out: process( clk, rst )
 begin
	if( rst = '1' ) then
		ack_uart <= '0';
		send_uart <= '0';
		btns <= (others => '0');
	elsif( clk'event and clk = '1' and write_strobe = '1' ) then
		-- port and bit index definitions in assembler code
		if( port_id(5) = '1' ) then
			send_uart <= out_port(0);
			ack_uart <= out_port(1);
			btns <= out_port(3 downto 2);
		end if;
		if( port_id(4) = '1' ) then data_tx <= out_port; end if;
		if( port_id(6) = '1' ) then data_mul <= out_port; end if;
	end if;
 end process;

-- multiplexing used input ports
in_port <= data_rx when port_id(0) = '1' else
	(0 => rdy_uart, 1 => valid, 7 => act_uart, others => '0') when port_id(1) = '1' else
	vys(15 downto 8) when port_id(2) = '1' else
	vys( 7 downto 0) when port_id(3) = '1' else
	(others => '0');

end architecture;

