-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : top.vhd
-- Revision : V1.2
-- Author :   Zdenk Sojka
-- Created :  04. 05. 2007
--
-- Modifications
-- 1.2	17. 05. 2007 by Zdenk Sojka
-- 1.1	14. 05. 2007 by Zdenk Sojka, Josef Nouzk, Jan Skalick
-- 1.0	04. 05. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- adi, obsahuje datovou a dc st a vstupn buffer
--
-- pi nastaven asovch podmnek T=18.3 ns
-- je nalezen perioda 18.098 ns ( 55.255 MHz )
--		nejpomalej spojen je nkde uvnit pblaze
--
--	vstupy	: rx na serial
-- vstupy	: tx ze serial
--
-- 1.2	odstrann vstup na LED, prava itelnosti kdu
-- 1.1	prava spoj mezi komponentami, pidan a zmnn signly
--			debugovac vstup na LED
--			formln pidn IBUFG kvli kompilaci
-- 1.0	prvn verze
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--	obsahuje IBUFG
library UNISIM;
use UNISIM.vcomponents.all;

entity top is
	port (
		rx : in std_logic;			-- data ze serial
		tx : out std_logic;			-- data na serial
		clk : in std_logic;			-- hodiny
		rst : in std_logic);			-- reset
end top;

architecture struct of top is

signal data_rx : std_logic_vector(7 downto 0);		-- vstup z UART
signal ack_uart : std_logic;								-- potvrzeni prijmu dat UART
signal rdy_uart : std_logic;								-- prijmuta data UART

signal data_tx : std_logic_vector(7 downto 0);		-- vystup do UART
signal send_uart : std_logic;								-- strobovani pro odeslani UART
signal act_uart : std_logic;								-- aktivni odesilani UART

signal data_mul : std_logic_vector(7 downto 0);		-- vystup do nasobicky
signal btns : std_logic_vector(1 downto 0);			-- tlactka k potvrzovani operandu nasobicky

signal valid : std_logic;									-- potvrzeni platnosti dat na vystupu nasobicky
signal vys : std_logic_vector(15 downto 0);			-- souin		

signal rst_b : std_logic;

component ser_in is
	port (
		rx : in std_logic;								-- seriova data
		ack : in std_logic;								-- smazat priznak rdy
		data : out std_logic_vector(7 downto 0);	-- prijata paralelni data
		rdy : out std_logic;								-- priznak prijatych dat

		clk : in std_logic;
		rst : in std_logic);
end component;

component ser_out is
	port (
		data : in std_logic_vector(7 downto 0);	-- data pro doeslani
		send : in std_logic;								-- prikaz pro odeslani
		act : out std_logic;								-- priznak aktivity
		tx : out std_logic;								-- data na seriovou linku

		clk : in std_logic;
		rst : in std_logic);
end component;

component bridge is
	port (
		data_rx : in std_logic_vector(7 downto 0);   -- vstup z UART
		data_tx : out std_logic_vector(7 downto 0);   -- vystup do UART
		data_mul : out std_logic_vector(7 downto 0);   -- vystup do nasobicky
		act_uart : in std_logic;   -- aktivni odesilani UART
		ack_uart : out std_logic;   -- potvrzeni prijmu dat UART
		send_uart : out std_logic;   -- strobovani pro odeslani  UART
		rdy_uart : in std_logic;   -- prijmuta data UART
		btns : out std_logic_vector(1 downto 0);	-- tlactka k potvrzovani operandu nasobicky
		valid : in std_logic;   -- potvrzeni platnosti dat na vystupu nasobicky
		vys : in std_logic_vector(15 downto 0);	-- souin		
		
		clk : in std_logic;
		rst : in std_logic);
end component;

component proc is
	port (
		btns : in std_logic_vector(1 downto 0);	-- tlatka z DIO
		swtch : in std_logic_vector(7 downto 0);	-- switche z DIO

		numA : out std_logic_vector(7 downto 0);	-- prvn initel
		numB : out std_logic_vector(7 downto 0);	-- druh initel
		vys : out std_logic_vector(15 downto 0);	-- souin
		valid : out std_logic;							-- platny vysledek

		clk : in std_logic;
		rst : in std_logic);
end component;

begin

-- IBUFG: Single-ended Input Buffer
-- All devices
-- Xilinx HDL Libraries Guide version 8.1i
IBUFG_inst : IBUFG
	generic map (
	IOSTANDARD => "DEFAULT")
	port map (
		O => rst_b,		-- Buffer output
		I => rst			-- Buffer input (connect directly to top-level port)
	);
-- End of IBUF_inst instantiation

SIN : ser_in		-- vstup ze seriove linky
	port map (
		rx => rx, data => data_rx, rdy => rdy_uart, ack => ack_uart, clk => clk, rst => rst_b );

SOUT : ser_out	-- vystup na seriovou linku
	port map (
	tx => tx, data => data_tx, act => act_uart, send => send_uart, clk => clk, rst => rst_b );

P1 : bridge port map (		-- obsluha komunikace s PC a s nasobickou
	data_rx => data_rx, data_tx => data_tx, data_mul => data_mul, act_uart => act_uart , ack_uart => ack_uart,
	send_uart => send_uart, rdy_uart => rdy_uart, btns => btns, valid => valid, vys => vys, clk => clk, rst => rst_b );

P2 : proc port map (			-- nasobicka
	btns => btns, swtch => data_mul, vys => vys, valid => valid, clk => clk, rst => rst_b );

end struct;

