-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : proc.vhd
-- Revision : V1.0
-- Author :   Jan Skalick
-- Created :  13. 05. 2007
--
-- Modifications
-- 1.0	13. 05. 2007 by Jan Skalick
-----------------------------------------------------------------
-- realizace nsobcho jdra uitm picoBlaze softcore
--
-- nejlep dosaen perioda je 15.703ns ( 63.682 MHz )
--
--	vstupy	: jako pedchoz realizace mikrokdem
-- vstupy	: jako pedchoz realizace mikrokdem
--
-- 1.0	initial release
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity proc is
	port (
		btns : in std_logic_vector(1 downto 0);	-- tlatka z DIO
		swtch : in std_logic_vector(7 downto 0);	-- switche z DIO

		numA : out std_logic_vector(7 downto 0);	-- prvn initel
		numB : out std_logic_vector(7 downto 0);	-- druh initel
		vys : out std_logic_vector(15 downto 0);	-- souin
		valid : out std_logic;							-- platny vysledek

		clk : in std_logic;
		rst : in std_logic);
		
end entity;

architecture rtl of proc is

signal port_id, out_port, in_port : std_logic_vector(7 downto 0);
signal write_strobe, read_strobe : std_logic;

component embedded_kcpsm is
    Port (      port_id : out std_logic_vector(7 downto 0);
           write_strobe : out std_logic;
            read_strobe : out std_logic;
               out_port : out std_logic_vector(7 downto 0);
                in_port : in std_logic_vector(7 downto 0);
              interrupt : in std_logic;
                  reset : in std_logic;
                    clk : in std_logic);
end component;

begin

	nasobicka : embedded_kcpsm
	port map (
		port_id => port_id,
		write_strobe => write_strobe,
		read_strobe => read_strobe,
		out_port => out_port,
		in_port => in_port,
		interrupt => '0',
		reset => rst,
		clk => clk
	);
	
-- buffering used output ports
data_out: process( clk, rst )
 begin
	if( rst = '1' ) then
		vys  <= (others => '0');
		numA <= (others => '0');
		numB <= (others => '0');
		valid <= '0';
	elsif( clk'event and clk = '1' and write_strobe = '1' ) then
		-- port and bit index definitions in assembler code
		if( port_id(1) = '1' ) then vys(7 downto 0)  <= out_port; end if;
		if( port_id(2) = '1' ) then vys(15 downto 8) <= out_port; end if;
		if( port_id(4) = '1' ) then valid <= out_port(1); end if;
		if( port_id(5) = '1' ) then numA <= out_port; end if;
		if( port_id(6) = '1' ) then numB <= out_port; end if;
	end if;
 end process;

-- multiplexing used input ports
in_port <= swtch when port_id(0) = '1' else
	(0 => btns(0), 1 => btns(1), others => '0') when port_id(3) = '1' else
	(others => '0');

end architecture;

