-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : ser_in.vhd
-- Revision : V1.2
-- Author :   Zdenk Sojka
-- Created :  03. 05. 2007
--
-- Modifications
-- 1.2	17. 05. 2007 by Zdenk Sojka
-- 1.1	14. 05. 2007 by Zdenk Sojka
-- 1.0	03. 05. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- seriovo - paraleln pevad
--	ek na start bit, pak strobuje data v polovin kadho bitu
--		- odolnost vi nepesnmu asovn
-- pracuje s rychlost 19200bps
-- po pijet jednoho byte je vstup 
--
-- 19200,8,N,1, bez rizeni
--
--	vstupy	: rx ze serial
--				  ack pro smazn pznaku pijatch dat
-- vstupy	: pijat byte dat
--				  pznak rdy pijat data
--
-- 1.2	prava itelnosti kdu
-- 1.1	opraveno zpracovn signlu 'ack'
--			drobn pravy
--			zvena rychlost na 19200 z 300bps
--			odstranno dvojit bufferovn nkterch signl
-- 1.0	prvn verze
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ser_in is
	port (
		rx : in std_logic;								-- seriova data
		ack : in std_logic;								-- smazat priznak rdy
		data : out std_logic_vector(7 downto 0);	-- prijata paralelni data
		rdy : out std_logic;								-- priznak prijatych dat

		clk : in std_logic;
		rst : in std_logic);
end ser_in;

architecture rtl of ser_in is

signal act : std_logic;
signal act_nxt : std_logic;
signal ack_i : std_logic;
signal rx_i : std_logic;
signal rx_prv : std_logic;
signal pos : std_logic_vector(8 downto 0);
signal pos_nxt : std_logic_vector(8 downto 0);
signal data_i : std_logic_vector(7 downto 0);
signal data_nxt : std_logic_vector(7 downto 0);
signal buff : std_logic_vector(7 downto 1);		-- nejnizsi bity nejsou nikdy vyuzity
signal buff_nxt : std_logic_vector(7 downto 1);
signal rdy_i : std_logic;
signal rdy_nxt : std_logic;
signal cnt : std_logic_vector(11 downto 0);
signal cnt_nxt : std_logic_vector(11 downto 0);


begin

	zachyceni : process ( rst, clk )						-- vstupni buffery

	begin
		if ( rst = '1' ) then
			rx_i <= '1';
			ack_i <= '1';
		elsif ( clk'event ) and ( clk = '1' ) then
			rx_i <= rx;
			ack_i <= ack;
		end if;
	end process;

	prijem : process ( rst, clk )							-- reseni celeho prijmu

	begin
		if ( rst = '1' ) then
			act_nxt <= '0';
			pos_nxt <= "100000000";
		elsif ( clk'event ) and ( clk = '1' ) then	-- nabezna hrana hodin
			if ( act = '0' ) then
				if	( rx_prv = '1' ) and ( rx_i = '0' ) then	-- neaktivni, prichod start bitu
					act_nxt <= '1';
					cnt_nxt <= X"516";
				end if;
			else
				if ( cnt = X"A2C" ) then
					if ( pos(0) = '1' ) then		-- vybubla jednicka, konec prenosu
						data_nxt <= rx_i & buff(7 downto 1);	-- nebude se kontrolovat parita
						act_nxt <= '0';
						pos_nxt <= "100000000";
					else
					   buff_nxt <= rx_i & buff(7 downto 2);
						act_nxt <= '1';
						pos_nxt <= rx_i & pos(8 downto 1);		-- posuvny registr doprava
						cnt_nxt <= (others=>'0');
					end if;
				else
					cnt_nxt <= cnt + '1';
				end if;
			end if;
		end if;
	end process;


	rdy_nxt <= '0' when rst = '1'
		else '1' when pos(0) = '1' and cnt = X"A2C"
		else ( rdy_i and not ack_i );

	preposlani : process ( rst, clk )						-- registrovani dalsich promennych

	begin
		if ( rst = '1' ) then
--			data_i <= data_nxt;
			rdy_i <= '0';
			rx_prv <= '1';
		elsif ( clk'event ) and ( clk = '1' ) then
			rdy_i <= rdy_nxt;									-- rdy je jen kombinacni logika, nastavit novou hodnotu
			rx_prv <= rx_i;									-- predchozi hodnota rx
		end if;
	end process;

	act <= act_nxt;
	pos <= pos_nxt;
	cnt <= cnt_nxt;
	buff <= buff_nxt;
	data_i <= data_nxt;

	rdy <= rdy_i;
	data <= data_i;

end rtl;

