-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : ser_out.vhd
-- Revision : V1.2
-- Author :   Zdenk Sojka
-- Created :  03. 05. 2007
--
-- Modifications
-- 1.2	17. 05. 2007 by Zdenk Sojka
-- 1.1	14. 05. 2007 by Zdenk Sojka
-- 1.0	03. 05. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- paraleln - seriov pevad
-- na signlu 'send' nate data a zane je poslat
-- odesln zane okamit, resetuje se asova
-- prvn jde start bit, pak data, dva stop bity
-- pokud je odesln aktivn, je signl 'act' a na signl 'send' nereaguje
--
-- 19200,8,N,2, bez rizeni
--
--	vstupy	: data a send pro zen odesln
-- vstupy	: tx na serial
--				  pznak act kdy je odesln aktivn
--
-- 1.2	prava itelnosti kdu
-- 1.1	zvena rychlost na 19200 z 300bps
--			drobn pravy
-- 1.0	prvn verze
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ser_out is
	port (
		data : in std_logic_vector(7 downto 0);	-- data pro doeslani
		send : in std_logic;								-- prikaz pro odeslani
		act : out std_logic;								-- priznak aktivity
		tx : out std_logic;								-- data na seriovou linku

		clk : in std_logic;
		rst : in std_logic);
end ser_out;

architecture rtl of ser_out is

signal act_i, act_nxt : std_logic;							-- signaly souvisejici s 'act'
signal cnt, cnt_nxt : std_logic_vector(11 downto 0);	-- citac pro casovani odesilanych dat
signal pos, pos_nxt : std_logic_vector(8 downto 0);	-- odesilana data vcetne start a stop bitu
signal ode, ode_nxt : std_logic_vector(3 downto 0);	-- citac odeslanych bitu
signal tx_i, tx_nxt : std_logic;								-- signaly souvisejici s 'tx'

signal data_i : std_logic_vector(7 downto 0);			-- bufferovane vstupy
signal send_i : std_logic;

begin

	vstupy : process ( rst, clk )						-- bufferovani vstupu
	
	begin
		if ( rst = '1' ) then
			send_i <= '0';
		elsif ( clk'event ) and ( clk = '1' ) then
			send_i <= send;
			data_i <= data;
		end if;
	end process;


	odeslani : process ( rst, clk )					-- nastaveni vystupnich dat, zmena hodnot citacu

	begin
		if ( rst = '1' ) then
			pos_nxt <= (others=>'1');
			tx_nxt <= '1';
			act_nxt <= '0';
		elsif ( clk'event ) and ( clk = '1' ) then	-- nabezna hrana hodin
			if ( act_i = '0' ) then		-- neni odesilani aktivni
				if	( send_i = '1' ) then	-- prikaz k poslani dat
					act_nxt <= '1';
					cnt_nxt <= (others=>'0');
					pos_nxt <= data_i & '0';	-- odesila se od nejnizsiho bitu - '0' je start bit, stop bit se doplni sam
					ode_nxt <= "1010";		-- 10 ( 2 stop bity )
				end if;
			else	-- aktivni odesilani
				if ( cnt = X"A2C" ) then
					tx_nxt <= pos(0);		-- odeslani nejnizsiho bitu
					pos_nxt <= '1' & pos(8 downto 1);	-- jednicky zleva - stop bity
					cnt_nxt <= (others=>'0');	-- reset citace
					if ( ode = 0 ) then		-- konec prenosu
						act_nxt <= '0';
					else
						ode_nxt <= ode - '1';
					end if;
				else
					cnt_nxt <= cnt + '1';
				end if;
			end if;
		end if;
	end process;

	pos <= pos_nxt;
	cnt <= cnt_nxt;
	ode <= ode_nxt;

	act_i <= act_nxt;
	tx_i <= tx_nxt;

	vystupy : process ( rst, clk )						-- bufferovani vystupu
	
	begin
		if ( rst = '1' ) then
			tx <= '1';
			act <= '0';
		elsif ( clk'event ) and ( clk = '1' ) then
			act <= act_i;
			tx <= tx_i;
		end if;
	end process;

end rtl;

