-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : naddr.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  20. 04. 2007
--
-- Modifications
-- 2.0	04. 05. 2007 by Zdenk Sojka
-- 1.0	22. 04. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- logika vpotu adresy dal mikroinstrukce
-- registrovan vstup
-- 
-- vstupy	: zkladn adresa dal mikroinstrukce
--				  kd vbru podmnky
--				  stavov signly
-- vstupy	: adresa dal mikroinstrukce
--
-- 2.0	zmna kdovn podmnek
-- 1.0	prava itelnosti kdu
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity naddr is
	port (
		naddr	: in std_logic_vector(2 downto 0);	-- zkladn adresa dal mikroinstrukce
		vp		: in std_logic_vector(1 downto 0);	-- vbr podmnky
		btnA	: in std_logic;							-- stisk tlatka A
		btnB	: in std_logic;							-- -//- B
		zero0 : in std_logic;							-- nulov posledn bit registru A
		zeroA : in std_logic;							-- nulov registr A

		addr	: out std_logic_vector(2 downto 0);	-- uren adresa dal mikroinstrukce

		rst : in std_logic;
		clk : in std_logic);
end naddr;

architecture rtl of naddr is

signal addr_nxt : std_logic_vector(2 downto 0);

begin

	urcit : process ( rst, naddr, btnA, btnB, zero0, zeroA ) is
	
	begin
		if ( rst = '1' ) then
			addr_nxt <= (others=>'0');
		else
			addr_nxt(2) <= naddr(2);
			case vp is
				when "01" =>
					addr_nxt(1) <= zero0;
					addr_nxt(0) <= zeroA;
				when "10" =>
					addr_nxt(1) <= naddr(1);
					addr_nxt(0) <= btnA;
				when "11" =>
					addr_nxt(1) <= naddr(1);
					addr_nxt(0) <= btnB;
				when others =>
					addr_nxt(1 downto 0) <= naddr(1 downto 0);
			end case;
		end if;
	end process;


	presun : process ( clk, rst, addr_nxt ) is
	
	begin
		if ( rst = '1' ) then
			addr <= (others=>'0');
		elsif ( clk = '1' ) and clk'event then
			addr <= addr_nxt;
		end if;
	end process;



end rtl;

