-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : proc.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  20. 04. 2007
--
-- Modifications
-- 2.0	04. 05. 2007 by Zdenk Sojka
-- 1.0	22. 04. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- adi, obsahuje datovou a dc st a vstupn buffer
--
--	vstupy	: vstupy od uivatele
-- vstupy	: data pro zobrazen na LCD a sseg
--
-- 2.0	odstrann signl 'load', pidn 'valid'
-- 1.0	prava itelnosti kdu
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity proc is
	port (
		btns : in std_logic_vector(1 downto 0);	-- tlatka z DIO
		swtch : in std_logic_vector(7 downto 0);	-- switche z DIO

		numA : out std_logic_vector(7 downto 0);	-- prvn initel
		numB : out std_logic_vector(7 downto 0);	-- druh initel
		vys : out std_logic_vector(15 downto 0);	-- souin
		valid : out std_logic;							-- platny vysledek

		clk : in std_logic;
		rst : in std_logic);
		
end proc;

architecture rtl of proc is

signal zeroA, zero0, setA, setB, shrA, shlB, k1, k2, resV, load, btnA, btnB : std_logic;
signal sw : std_logic_vector(7 downto 0);

component buff is
	port (
		btns : in std_logic_vector(1 downto 0);
		swtch : in std_logic_vector(7 downto 0);

		btnA : out std_logic;
		btnB : out std_logic;
		sw : out std_logic_vector(7 downto 0);

		clk : in std_logic;
		rst : in std_logic);
end component;

component ctrl
	port (
		zeroA : in std_logic;							-- priznak - registr A nulovy
		zero0 : in std_logic;							-- priznak - posledni bit registru A nulovy

		btnA : in std_logic;								-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		btnB : in std_logic;								-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		
		setA : out std_logic;							-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		setB : out std_logic;							-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		shrA : out std_logic;							-- prikaz - regA >> 1
		shlB : out std_logic;							-- prikaz - regB << 1
		resV : out std_logic;							-- prikaz - nulovani registru s vysledkem
		
		valid : out std_logic;							-- platna data na vystupu

		k1 : out std_logic;								-- scitani, prvni krok
		k2 : out std_logic;								-- druhy krok scitani - signal pro scitacku pro zachovani priznaku
		
		clk : in std_logic;
		rst : in std_logic
	);
end component;

component data
	port (
		sw : in std_logic_vector(7 downto 0);		-- 8 bit hodnota pri zapisu do registru (nastaveni switchu)

		setA : in std_logic;								-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		setB : in std_logic;								-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		shrA : in std_logic;								-- prikaz - regA >> 1
		shlB : in std_logic;								-- prikaz - regB << 1
		resV : in std_logic;								-- prikaz - nulovani registru s vysledkem
		
		k1 : in std_logic;								-- prikaz - scitani, prvni krok
		k2 : in std_logic;								-- prikaz - druhy krok scitani - signal pro scitacku pro zachovani priznaku

      numA : out std_logic_vector(7 downto 0);	-- prvni cislo pro zobrazeni na 8seg
      numB : out std_logic_vector(7 downto 0);	-- druhe cislo -//-
      vys : out std_logic_vector(15 downto 0);	-- soucin pro zobrazeni na LCD

		zeroA : out std_logic;							-- priznak - registr A nulovy
		zero0 : out std_logic;							-- priznak - posledni bit registru A nulovy

		clk : in std_logic;
		rst : in std_logic
	);
end component;

begin

	bafr : buff
	port map (
		btns => btns(1 downto 0),
		swtch => swtch,
		btnA => btnA,
		btnB => btnB,
		sw => sw,
		rst => rst,
		clk => clk
	);

	radic : ctrl port map (
		zeroA => zeroA, zero0 => zero0, btnA => btnA, setA => setA,
		btnB => btnB, setB => setB, shrA => shrA, shlB => shlB,
		resV => resV, valid => valid, k1 => k1, k2 => k2,
		clk => clk, rst => rst
		);

	datcast : data port map (
		zeroA => zeroA, zero0 => zero0, setA => setA, shrA => shrA,
		setB => setB, shlB => shlB,
		resV => resV, k1 => k1, k2 => k2,
		sw => sw,
		numA => numA, numB => numB, vys => vys,
		clk => clk, rst => rst
		);


end rtl;

