-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : proc_bench.vhd
-- Revision : V1.0
-- Author :   Jan Skalicky
-- Created :  22. 04. 2007
--
-- Modifications
--
-- 2.0	04. 05. 2007 by Zdenk Sojka
-- 
-----------------------------------------------------------------
-- test bench for this entity
--
-- 2.0	zmna chovn, zadvn sel jednotliv, ekn na signl 'valid'
--			ovena funkce upravenho nvrhu
-- 1.0
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY proc_bench_vhd IS
END proc_bench_vhd;

ARCHITECTURE behavior OF proc_bench_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT proc
	PORT(
		btns : IN std_logic_vector(1 downto 0);
		swtch : IN std_logic_vector(7 downto 0);
		clk : IN std_logic;
		rst : IN std_logic;          
		valid : OUT std_logic;
		numA : OUT std_logic_vector(7 downto 0);
		numB : OUT std_logic_vector(7 downto 0);
		vys : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;

	--Inputs
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL btns :  std_logic_vector(1 downto 0) := (others=>'0');
	SIGNAL swtch :  std_logic_vector(7 downto 0) := (others=>'0');

	--Outputs
	SIGNAL numA :  std_logic_vector(7 downto 0);
	SIGNAL numB :  std_logic_vector(7 downto 0);
	SIGNAL vys :  std_logic_vector(15 downto 0);
	SIGNAL valid :  std_logic;

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: proc PORT MAP(
		btns => btns,
		swtch => swtch,
		numA => numA,
		numB => numB,
		vys => vys,
		valid => valid,
		clk => clk,
		rst => rst
	);

	tb : process
	 begin

		rst <= '1';
		wait for 10ns;
		rst <= '0';

		for i in 0 to 255 loop
			for j in 0 to 255 loop
				wait until clk'event and clk = '0';
				swtch<=std_logic_vector(to_unsigned(i,8));
				btns(0)<='1';
				wait until clk'event and clk = '0';
				btns(0)<='0';
				wait until clk'event and clk = '0';

				swtch<=std_logic_vector(to_unsigned(j,8));
				btns(1)<='1';
				wait until clk'event and clk = '0';
				btns(1)<='0';
				wait until clk'event and clk = '0';
				wait until valid = '1';

				assert (vys=std_logic_vector(to_unsigned(i*j,16)));
			end loop;
		end loop;


		report "Testbench finished, check for failed assertions" severity note;
		wait;
	 end process;
	
	clock : process
	 begin
		clk <= '0';
		wait for 100ns;
		clk <= '1';
		wait for 100ns;
	 end process;

END;
