-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : registrA_bench.vhd
-- Revision : V1.0
-- Author :   Jan Skalicky
-- Created :  22. 04. 2007
--
-- Modifications
-- 
-----------------------------------------------------------------
-- test bench for this entity
--
-- 1.0
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY registrA_bench_vhd IS
END registrA_bench_vhd;

ARCHITECTURE behavior OF registrA_bench_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT registrA
	PORT(
		sw : IN std_logic_vector(7 downto 0);
		set : IN std_logic;
		shr : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		zero : OUT std_logic;
		zero0 : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL set :  std_logic := '0';
	SIGNAL shr :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL sw :  std_logic_vector(7 downto 0) := (others=>'0');

	--Outputs
	SIGNAL zero :  std_logic;
	SIGNAL zero0 :  std_logic;
	
BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: registrA PORT MAP(
		sw => sw,
		set => set,
		shr => shr,
		zero => zero,
		zero0 => zero0,
		clk => clk,
		rst => rst
	);

	tb : process
	 begin

		rst <= '1';
		wait for 10ns;
		rst <= '0';

		-- testing reset - 8 zero bits should go out
		wait until clk'event and clk = '0';
		assert( zero = '1' ); 
		assert( zero0 = '1' ); 		
		wait until clk'event and clk = '0';
		shr <= '1';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 		
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		wait until clk'event and clk = '0';
		assert( zero0 = '1' ); 
		
		-- testing certain data with various level widths
		wait until clk'event and clk = '0';
		sw <= "10001101";
		set <= '1';
		shr <= '0';
		wait until clk'event and clk = '0';
		assert( zero0 = '0' );
		wait until clk'event and clk = '0';
		set <= '0';
		shr <= '1';
		wait for 1ps;   -- wait for shift to come through
		assert( zero0 = '1' );
		wait until clk'event and clk = '0';
		assert( zero0 = '0' );
		wait until clk'event and clk = '0';
		assert( zero0 = '0' );
		wait until clk'event and clk = '0';
		assert( zero0 = '1' );
		wait until clk'event and clk = '0';
		assert( zero0 = '1' );
		wait until clk'event and clk = '0';
		assert( zero0 = '1' );
		wait until clk'event and clk = '0';
		assert( zero0 = '0' );
		wait until clk'event and clk = '0';
		assert( zero0 = '1' );
		wait until clk'event and clk = '0';
		-- now, registr is empty (filled with 0)
		shr <= '0';
		assert( zero0 = '1' );

		report "Testbench finished, check for failed assertions" severity note;
		wait;
	 end process;
	
	clock : process
	 begin
		clk <= '0';
		wait for 100ns;
		clk <= '1';
		wait for 100ns;
	 end process;
	
END;
