-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : registrAtb.vhd
-- Revision : V1.1
-- Author :   Josef Nouzk
-- Created :  22. 04. 2007
--
-- Modifications
-- 1.1	22. 04. 2007 by Zdenk Sojka
-- 1.0	22. 04. 2007 by Josef Nouzk
-----------------------------------------------------------------
-- testbench pro registr A - zkou natn a posun
--
-- 1.1	upraveno asovn
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY registrAtb_vhd IS
END registrAtb_vhd;

ARCHITECTURE behavior OF registrAtb_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT registrA
	PORT(
		sw : IN std_logic_vector(7 downto 0);
		set : IN std_logic;
		shr : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		zero : OUT std_logic;
		zero0 : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL set :  std_logic := '0';
	SIGNAL shr :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL sw :  std_logic_vector(7 downto 0) := (others=>'0');

	--Outputs
	SIGNAL zero :  std_logic;
	SIGNAL zero0 :  std_logic;

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: registrA PORT MAP(
		sw => sw,
		set => set,
		shr => shr,
		zero => zero,
		zero0 => zero0,
		clk => clk,
		rst => rst
	);

	process
		begin
			clk<='0';
			wait for 10ns;
			clk<='1';
			wait for 10ns;
	end process;

	tb : PROCESS
	
	variable err_cnt: integer :=0;
	
	BEGIN

		-- Wait 100 ns for global reset to finish
		wait for 100 ns;
		
		rst <= '1';
		wait for 40ns;
		rst <= '0';
		wait for 40ns;
		-- Place stimulus here
		---case1-----nacteni
		wait for 40ns;
		sw <="00000010";
		wait for 80ns;
		set <= '1';
		wait for 100ns;
		set <= '0';
		wait for 40ns;
		assert (zero='0') report "load Error!" severity error;
		if (zero/='0') then
			err_cnt:=err_cnt+1;
		end if;
		
		---case2------shr
		wait for 40ns;
		shr <='1';
		wait for 40ns;
		shr<='0';
		wait for 40ns;
		shr <= '1';
		wait for 40ns;
		shr <= '0';
		wait for 40ns;
		assert (zero='1') report "shr Error!" severity error;
		if (zero/='1') then
			err_cnt:=err_cnt+1;
		end if;
		
		-- summary of testbench
		if (err_cnt=0) then
	    assert false
	    report "Testbench of Adder completed successfully!"
	    severity note;
		else
	    assert true
	    report "Something wrong, try again"
	    severity error;
		end if;
				
		wait; -- will wait forever
	END PROCESS;

END;
